import { parseArgs } from "node:util";
import { titleCase } from "title-case";
import { withSpinner } from "../cli/spinners.js";
import { allArgOptions } from "./args.js";
import { augmentOptionsWithExcludes } from "./augmentOptionsWithExcludes.js";
import { createOptionDefaults } from "./createOptionDefaults/index.js";
import { detectEmailRedundancy } from "./detectEmailRedundancy.js";
import { ensureRepositoryExists } from "./ensureRepositoryExists.js";
import { getBase } from "./getBase.js";
import { getGitHub } from "./getGitHub.js";
import { getPrefillOrPromptedOption } from "./getPrefillOrPromptedOption.js";
import { logInferredOptions } from "./logInferredOptions.js";
import { optionsSchema } from "./optionsSchema.js";
async function readOptions(args, mode, promptedOptions = {}) {
  const defaults = createOptionDefaults(promptedOptions);
  const { values } = parseArgs({
    args,
    options: allArgOptions,
    strict: false,
    tokens: true
  });
  if (mode === "migrate" && !values.base) {
    values.base = await getBase();
  }
  const mappedOptions = {
    access: values.access,
    author: values.author,
    auto: !!values.auto,
    base: values.base,
    bin: values.bin,
    description: values.description,
    directory: values.directory,
    email: values.email ?? values["email-github"] ?? values["email-npm"] ? {
      github: values.email ?? values["email-github"],
      npm: values.email ?? values["email-npm"]
    } : void 0,
    excludeAllContributors: values["exclude-all-contributors"],
    excludeCompliance: values["exclude-compliance"],
    excludeLintDeprecation: values["exclude-lint-deprecation"],
    excludeLintESLint: values["exclude-lint-eslint"],
    excludeLintJSDoc: values["exclude-lint-jsdoc"],
    excludeLintJson: values["exclude-lint-json"],
    excludeLintKnip: values["exclude-lint-knip"],
    excludeLintMd: values["exclude-lint-md"],
    excludeLintPackageJson: values["exclude-lint-package-json"],
    excludeLintPackages: values["exclude-lint-packages"],
    excludeLintPerfectionist: values["exclude-lint-perfectionist"],
    excludeLintRegex: values["exclude-lint-regex"],
    excludeLintSpelling: values["exclude-lint-spelling"],
    excludeLintStrict: values["exclude-lint-strict"],
    excludeLintYml: values["exclude-lint-yml"],
    excludeReleases: values["exclude-releases"],
    excludeRenovate: values["exclude-renovate"],
    excludeTests: values["unit-tests"],
    funding: values.funding,
    guide: values.guide,
    logo: values.logo,
    offline: values.offline,
    owner: values.owner,
    preserveGeneratedFrom: values["preserve-generated-from"] ?? values.owner === "JoshuaKGoldberg",
    repository: values.repository,
    skipAllContributorsApi: values["skip-all-contributors-api"] ?? values.offline,
    skipGitHubApi: values["skip-github-api"] ?? values.offline,
    skipInstall: values["skip-install"],
    skipRemoval: values["skip-removal"],
    skipRestore: values["skip-restore"],
    skipUninstall: values["skip-uninstall"],
    title: values.title
  };
  const emailError = detectEmailRedundancy(values);
  if (emailError) {
    return {
      cancelled: true,
      error: emailError,
      options: mappedOptions
    };
  }
  const optionsParseResult = optionsSchema.safeParse(mappedOptions);
  if (!optionsParseResult.success) {
    return {
      cancelled: true,
      error: optionsParseResult.error,
      options: mappedOptions
    };
  }
  const options = optionsParseResult.data;
  const ownerOption = await getPrefillOrPromptedOption(
    "owner",
    !!mappedOptions.auto,
    "What organization or user will the repository be under?",
    defaults.owner
  );
  options.owner ??= ownerOption.value;
  if (!options.owner) {
    return {
      cancelled: true,
      error: ownerOption.error,
      options
    };
  }
  const repositoryOption = await getPrefillOrPromptedOption(
    "repository",
    !!mappedOptions.auto,
    "What will the kebab-case name of the repository be?",
    defaults.repository
  );
  options.repository ??= repositoryOption.value;
  if (!options.repository) {
    return {
      cancelled: true,
      error: repositoryOption.error,
      options
    };
  }
  const { github, repository } = await ensureRepositoryExists(
    options.skipGitHubApi ? void 0 : await withSpinner("Checking GitHub authentication", getGitHub),
    {
      mode,
      owner: options.owner,
      repository: options.repository
    }
  );
  if (!repository) {
    return { cancelled: true, error: repositoryOption.error, options };
  }
  const descriptionOption = await getPrefillOrPromptedOption(
    "description",
    !!mappedOptions.auto,
    "How would you describe the new package?",
    async () => await defaults.description() ?? "A very lovely package. Hooray!"
  );
  options.description ??= descriptionOption.value;
  if (!options.description) {
    return { cancelled: true, error: descriptionOption.error, options };
  }
  const titleOption = await getPrefillOrPromptedOption(
    "title",
    !!mappedOptions.auto,
    "What will the Title Case title of the repository be?",
    async () => await defaults.title() ?? titleCase(repository).replaceAll("-", " ")
  );
  options.title ??= titleOption.value;
  if (!options.title) {
    return { cancelled: true, error: titleOption.error, options };
  }
  let guide;
  if (options.guide) {
    if (options.guideTitle) {
      guide = { href: options.guide, title: options.guideTitle };
    } else {
      const titleOption2 = await getPrefillOrPromptedOption(
        "getPrefillOrPromptedOption",
        !!mappedOptions.auto,
        "What is the title text for the guide?"
      );
      if (!titleOption2.value) {
        return { cancelled: true, error: titleOption2.error, options };
      }
      guide = { href: options.guide, title: titleOption2.value };
    }
  }
  let logo;
  if (options.logo) {
    if (options.logoAlt) {
      logo = { alt: options.logoAlt, src: options.logo };
    } else {
      const logoAltOption = await getPrefillOrPromptedOption(
        "getPrefillOrPromptedOption",
        !!mappedOptions.auto,
        "What is the alt text (non-visual description) of the logo?"
      );
      if (!logoAltOption.value) {
        return { cancelled: true, error: logoAltOption.error, options };
      }
      logo = { alt: logoAltOption.value, src: options.logo };
    }
  }
  let email = options.email ?? await defaults.email();
  if (!email) {
    const emailOption = await getPrefillOrPromptedOption(
      "email",
      !!mappedOptions.auto,
      "What email should be used in package.json and .md files?"
    );
    if (!emailOption.value) {
      return { cancelled: true, error: emailOption.error, options };
    }
    email = { github: emailOption.value, npm: emailOption.value };
  }
  const augmentedOptions = await augmentOptionsWithExcludes({
    ...options,
    access: options.access ?? "public",
    author: options.author ?? await defaults.owner(),
    description: options.description,
    directory: options.directory ?? promptedOptions.directory ?? options.repository,
    email,
    funding: options.funding ?? await defaults.funding(),
    guide,
    logo,
    mode,
    owner: options.owner,
    repository,
    title: options.title
  });
  if (!augmentedOptions) {
    return {
      cancelled: true,
      options
    };
  }
  if (options.auto) {
    logInferredOptions(augmentedOptions);
  }
  return {
    cancelled: false,
    github,
    options: augmentedOptions
  };
}
export {
  readOptions
};
//# sourceMappingURL=readOptions.js.map